import bpy


class POWERPROPS_OT_add_node(bpy.types.Operator):
    bl_idname = 'powerprops.add_node'
    bl_label = 'Add Node'
    bl_description = 'Add an attribute node to the active material'
    bl_options = {'REGISTER', 'UNDO', 'INTERNAL'}

    data_path: bpy.props.StringProperty(options={'HIDDEN', 'SKIP_SAVE'})
    property_name: bpy.props.StringProperty(options={'HIDDEN', 'SKIP_SAVE'})

    def invoke(self, context: bpy.types.Context, event: bpy.types.Event) -> set[str]:
        context.window_manager.modal_handler_add(self)
        return {'RUNNING_MODAL'}

    def modal(self, context: bpy.types.Context, event: bpy.types.Event):
        return self.execute(context)

    def execute(self, context: bpy.types.Context) -> set:
        object = context.active_object
        material = object.active_material

        if not material:
            self.report({'ERROR'}, f'"{object.name}" does not have an active material')
            return {'CANCELLED'}

        if not material.node_tree:
            self.report({'ERROR'}, f'"{material.name}" does not use nodes')
            return {'CANCELLED'}

        nodes = material.node_tree.nodes
        node: bpy.types.ShaderNodeAttribute = nodes.new(bpy.types.ShaderNodeAttribute.__name__)

        node.attribute_type = 'OBJECT'
        node.attribute_name = self.property_name

        for node in nodes:
            node.select = False

        node.select = True
        nodes.active = node

        self.report({'INFO'}, f'Added "{self.property_name}" node to "{material.name}"')
        return {'FINISHED'}
