import bpy

from .. import utils


class POWERPROPS_OT_add_property(bpy.types.Operator):
    bl_idname = 'powerprops.add_property'
    bl_label = 'Add Property'
    bl_description = '.\n'.join((
        'Add a new property to the active object',
        'Shift click to add the property to all selected objects',
    ))
    bl_options = {'REGISTER', 'UNDO', 'INTERNAL'}

    data_path: bpy.props.StringProperty(options={'HIDDEN', 'SKIP_SAVE'})

    to_selected: bpy.props.BoolProperty(
        name='To Selected',
        description='Add the property to all selected objects',
        default=False,
    )

    def invoke(self, context: bpy.types.Context, event: bpy.types.Event) -> set[str]:
        self.to_selected = event.shift

        context.window_manager.modal_handler_add(self)
        return {'RUNNING_MODAL'}

    def modal(self, context: bpy.types.Context, event: bpy.types.Event):
        return self.execute(context)

    def execute(self, context: bpy.types.Context) -> set:
        active_data: bpy.types.ID = context.path_resolve(self.data_path)

        temp = set(active_data.keys())
        bpy.ops.wm.properties_add(data_path=self.data_path)
        property_name = (set(active_data.keys()) - temp).pop()

        data_blocks = {active_data}

        if self.to_selected:
            for object in context.selected_objects:
                with context.temp_override(active_object=object):
                    object_data: bpy.types.ID = context.path_resolve(self.data_path)

                if object_data and property_name not in object_data:
                    object_data[property_name] = active_data[property_name]
                    data_blocks.add(object_data)

        utils.common.tag_redraw(context)
        for data_block in data_blocks:
            utils.common.update_tag(context, data_block)

        bpy.ops.powerprops.edit_property(
            'INVOKE_DEFAULT',
            data_path=self.data_path,
            property_name=property_name,
            edit_selected=self.to_selected,
        )

        info = f'{len(data_blocks)} data blocks' if self.to_selected else f'"{active_data.name}"'
        self.report({'INFO'}, f'Added "{property_name}" to {info}')
        return {'FINISHED'}
