import bpy

from .. import utils


class POWERPROPS_OT_drive_with_property(bpy.types.Operator):
    bl_idname = 'powerprops.drive_with_property'
    bl_label = 'Drive with Property'
    bl_description = 'Create a custom property on the active object which drives this property'
    bl_options = {'REGISTER', 'UNDO', 'INTERNAL'}

    @classmethod
    def poll(cls, context: bpy.types.Context):
        button_pointer: bpy.types.ID = getattr(context, 'button_pointer', None)
        button_prop: bpy.types.Property = getattr(context, 'button_prop', None)

        if not button_pointer:
            return False
        if not hasattr(button_pointer.id_data, 'animation_data'):
            return False

        try:
            eval(repr(button_pointer))
        except:
            return False

        if not button_prop:
            return False
        if button_prop.type not in ('BOOLEAN', 'INT', 'FLOAT'):
            return False

        if button_prop in button_pointer.bl_rna.properties.values():
            if not button_prop.is_animatable:
                return False

        return True

    def invoke(self, context: bpy.types.Context, event: bpy.types.Event) -> set[str]:
        self.button_pointer: bpy.types.ID = getattr(context, 'button_pointer', None)
        self.button_prop: bpy.types.Property = getattr(context, 'button_prop', None)

        return self.execute(context)

    def execute(self, context: bpy.types.Context) -> set:
        source_data: bpy.types.ID = self.button_pointer
        property: bpy.types.Property = self.button_prop

        wm_props = context.window_manager.powerprops
        target_data: bpy.types.ID = context.path_resolve(wm_props.data_path)

        target_data[property.name] = source_data.path_resolve(utils.property.get_path(source_data, property))
        target_data.id_properties_ui(property.name).update(**utils.property.to_ui_dict(property))

        if property.is_array:
            for index in range(len(target_data[property.name])):
                utils.property.drive(source_data, target_data, property, index)
        else:
            utils.property.drive(source_data, target_data, property)

        utils.common.tag_redraw(context)
        for data_block in (source_data, target_data):
            utils.common.update_tag(context, data_block)

        bpy.ops.powerprops.edit_property(
            'INVOKE_DEFAULT',
            data_path=wm_props.data_path,
            property_name=property.name,
            edit_selected=False,
        )

        info_a = f'"{property.name}"'
        info_b = f'"{getattr(target_data, "name", target_data.id_data.name)}"'
        info_c = f'"{getattr(source_data, "name", source_data.id_data.name)}"'
        self.report({'INFO'}, f'Added {info_a} to {info_b} and driver to {info_c}')
        return {'FINISHED'}
