import bpy

from . import add_node, copy_property, edit_property, randomize_property, remove_property, select_by_property

DESCRIPTIONS = [
    edit_property.POWERPROPS_OT_edit_property.bl_description,
    select_by_property.POWERPROPS_OT_select_by_property.bl_description,
    copy_property.POWERPROPS_OT_copy_property.bl_description,
    remove_property.POWERPROPS_OT_remove_property.bl_description,
    randomize_property.POWERPROPS_OT_randomize_property.bl_description,
    add_node.POWERPROPS_OT_add_node.bl_description,
]

TYPE_ITEMS = [
    ('EDIT', 'Edit', '', 'PREFERENCES', 0),
    ('SELECT', 'Select', '', 'RESTRICT_SELECT_OFF', 1),
    ('COPY', 'Copy', '', 'DUPLICATE', 2),
    ('REMOVE', 'Remove', '', 'X', 3),
    None,
    ('RANDOMIZE', 'Randomize', '', 'MOD_NOISE', 4),
    ('NODE', 'Node', '', 'MATERIAL', 5),
]


class PropertyMenuHelper:
    bl_label = 'Property Menu'
    bl_description = 'Perform an action with a property'
    bl_options = {'REGISTER', 'INTERNAL'}

    data_path: bpy.props.StringProperty(options={'HIDDEN', 'SKIP_SAVE'})
    property_name: bpy.props.StringProperty(options={'HIDDEN', 'SKIP_SAVE'})

    @classmethod
    def description(cls, context: bpy.types.Context, properties: dict):
        return DESCRIPTIONS[properties['type']]

    def execute(self, context: bpy.types.Context) -> set[str]:
        operator = {
            'EDIT': bpy.ops.powerprops.edit_property,
            'SELECT': bpy.ops.powerprops.select_by_property,
            'COPY': bpy.ops.powerprops.copy_property,
            'REMOVE': bpy.ops.powerprops.remove_property,
            'RANDOMIZE': bpy.ops.powerprops.randomize_property,
            'NODE': bpy.ops.powerprops.add_node,
        }[self.type]

        operator('INVOKE_DEFAULT', data_path=self.data_path, property_name=self.property_name)
        return {'FINISHED'}


class POWERPROPS_OT_property_menu_generic(bpy.types.Operator, PropertyMenuHelper):
    bl_idname = 'powerprops.property_menu_generic'
    type: bpy.props.EnumProperty(items=TYPE_ITEMS[:4])


class POWERPROPS_OT_property_menu_numeric(bpy.types.Operator, PropertyMenuHelper):
    bl_idname = 'powerprops.property_menu_numeric'
    type: bpy.props.EnumProperty(items=TYPE_ITEMS)
