import bpy

from .. import ui, utils


class PropertyProps(bpy.types.PropertyGroup):
    select: bpy.props.BoolProperty(
        name='Select',
        description='This property is affected by batch operations',
        default=False,
    )


class WMProps(bpy.types.PropertyGroup):

    def module(self) -> str:
        return utils.common.module()

    def prefs(self) -> bpy.types.AddonPreferences:
        return utils.common.prefs()

    def draw(self, layout: bpy.types.UILayout):
        ui.main_panel.draw(layout)

    data_path: bpy.props.EnumProperty(
        name='Data Path',
        items=[
            ('active_object', 'Object', 'Manage custom properties on the active object', 'OBJECT_DATA', 0),
            ('active_object.data', 'Data', 'Manage custom properties on the object data', 'MESH_DATA', 1),
        ],
        default='active_object',
    )

    properties: bpy.props.CollectionProperty(type=PropertyProps)

    def ensure_property(self, property_name: str) -> PropertyProps:
        if property_name in self.properties:
            return self.properties[property_name]

        property: PropertyProps = self.properties.add()
        property.name = property_name

        return property
