import bpy

from .. import utils


class POWERPROPS_PT_main(bpy.types.Panel):
    bl_label = 'PowerProps'
    bl_category = 'PowerSave'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'

    def draw(self, context: bpy.types.Context):
        draw(self.layout.column())


def draw(layout: bpy.types.UILayout):
    wm_props = bpy.context.window_manager.powerprops

    layout.use_property_split = True
    layout.use_property_decorate = False

    box = layout.box().column()

    row = box.row()
    row.use_property_split = False
    row.prop(wm_props, 'data_path', expand=True)

    data_path: str = wm_props.data_path
    data: bpy.types.ID = bpy.context.path_resolve(data_path)

    if not data:
        return

    layout.separator()

    box = layout.box().column()

    row = box.row()
    op = row.operator('powerprops.add_property', text='New', icon='ADD')
    op.data_path = data_path
    op = row.operator('powerprops.select_by_property', text='Select', icon='RESTRICT_SELECT_OFF')
    op.data_path = data_path
    op.batch = True

    row = box.row()
    op = row.operator('powerprops.copy_property', text='Copy', icon='DUPLICATE')
    op.data_path = data_path
    op.batch = True
    op = row.operator('powerprops.remove_property', text='Remove', icon='X')
    op.data_path = data_path
    op.batch = True

    if not data.keys():
        return

    layout.separator()

    box = layout.box().column()

    for property_name in sorted(data.keys()):
        property_value = utils.property.to_builtin(data[property_name])

        if isinstance(property_value, list):
            property_type = type(property_value[0] if property_value else None)
            invalid_list = len(property_value) not in range(1, 5)
        else:
            property_type = type(property_value)
            invalid_list = False

        try:
            ui_dict: dict = data.id_properties_ui(property_name).as_dict()
        except:
            ui_dict = None

        row = box.row(align=True)

        sub = row.row(align=True)
        sub.use_property_split = False
        sub.alignment = 'LEFT'
        property_props = wm_props.ensure_property(property_name)
        sub.prop(property_props, 'select', text='')

        if ui_dict and not invalid_list:
            sub = row.row(align=True)
            property = f'["{bpy.utils.escape_identifier(property_name)}"]'
            slider = ui_dict.get('soft_min') == 0 and ui_dict.get('soft_max') == 1
            sub.prop(data, property, slider=slider, translate=False)

        else:
            split = row.split(factor=0.4, align=True)

            sub = split.row(align=True)
            sub.alignment = 'RIGHT'
            sub.label(text=property_name, translate=False)

            sub = split.row(align=True)
            op = sub.operator('wm.properties_edit_value', text='Edit Value')
            op.data_path = data_path
            op.property_name = property_name

        sub = row.row(align=True)
        sub.alignment = 'RIGHT'

        if property_type in (int, float):
            op = sub.operator_menu_enum('powerprops.property_menu_numeric', 'type', text='')
        else:
            op = sub.operator_menu_enum('powerprops.property_menu_generic', 'type', text='')

        op.data_path = data_path
        op.property_name = property_name
