import random

import bpy

from .. import ui
from ... import __package__ as base_package

DATA_TYPES = {
    'AREALIGHT': 'LIGHT',
    'VECTORFONT': 'FONT',
    'LIGHTPROBE': 'LIGHT_PROBE',
    'FREESTYLELINESTYLE': 'LINESTYLE',
    'METABALL': 'META',
    'PARTICLESETTINGS': 'PARTICLE',
    'POINTLIGHT': 'LIGHT',
    'SPOTLIGHT': 'LIGHT',
    'SUNLIGHT': 'LIGHT',
    'SURFACECURVE': 'CURVE',
    'TEXTCURVE': 'CURVE',
}


def module() -> str:
    return base_package


def prefs() -> bpy.types.AddonPreferences:
    return bpy.context.preferences.addons[module()].preferences


def update_panel_category(self=None, context=None):
    panel = ui.main_panel.POWERPROPS_PT_main
    category = prefs().panel_category

    panel.bl_category = category
    panel.bl_region_type = 'UI' if category else 'HEADER'

    bpy.utils.unregister_class(panel)
    bpy.utils.register_class(panel)


def get_data_type(data_block: bpy.types.ID) -> str:
    data_type = type(data_block).__name__.upper()
    return DATA_TYPES.get(data_type, data_type)


def get_data_blocks() -> list[bpy.types.ID]:
    data_blocks = set()

    for key in dir(bpy.data):
        collection = getattr(bpy.data, key, None)

        if isinstance(collection, bpy.types.bpy_prop_collection):
            for data_block in collection:
                data_blocks.add(data_block)

                if getattr(data_block, 'node_tree', None):
                    data_blocks.add(data_block.node_tree)

    return list(data_blocks)


def tag_redraw(context: bpy.types.Context):
    for window in context.window_manager.windows:
        for area in window.screen.areas:
            area.tag_redraw()


def update_tag(context: bpy.types.Context, data: bpy.types.ID):
    if isinstance(data, bpy.types.Object):
        data.update_tag(refresh={'OBJECT'})

    else:
        for object in context.visible_objects:
            if object.data is data:
                object.update_tag(refresh={'DATA'})


def seeded_random(seed: int, min: int | float, max: int | float) -> int | float:
    rand = random.Random(seed)

    if min > max:
        min, max = max, min

    if isinstance(min, int):
        return rand.randint(min, max)
    else:
        return rand.uniform(min, max)
